<?xml version="1.0" encoding="ISO-8859-1"?>
<xsl:stylesheet version="1.0"
 xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
 xmlns:xsltc="http://xml.apache.org/xalan/xsltc"
 xmlns:redirect="http://xml.apache.org/xalan/redirect"
 extension-element-prefixes="redirect"
>
<xsl:param name="icon_width"  select="16"/>
<xsl:param name="icon_height" select="$icon_width"/>
<xsl:param name="actuator_line_color">#FF99CC</xsl:param>
<xsl:param name="teleporter_line_color">#3399FF</xsl:param>
<xsl:param name="strings_xml">../../strings/dm_en.xml</xsl:param>
<xsl:param name="output_path">.</xsl:param>
<xsl:param name="resources_path">.</xsl:param>

<xsl:variable name="doc_xml" select="document($strings_xml)"/>

<xsl:template match="dungeon">

<xsl:variable name="file" select="concat($output_path, '/dungeon.html')"/> 
<redirect:write file="{$file}">
  <html>
    <head>
      <title>Dungeon</title>
      <link rel="stylesheet" type="text/css">
        <xsl:attribute name="href"><xsl:value-of select="$resources_path"/>/styles/dungeon.css</xsl:attribute>
      </link>
    </head>
    <body>
      Dungeon maps<br/><br/>
      <xsl:for-each select="maps/map">
        <a><xsl:attribute name="href">map_<xsl:value-of select="@index+1"/>.html</xsl:attribute>
        map_<xsl:value-of select="@index+1"/>
        </a><br/>
      </xsl:for-each>
    </body>
  </html>
</redirect:write>

<xsl:apply-templates select="maps" />
</xsl:template>

<xsl:template match="maps">
  <xsl:for-each select="map">
    <xsl:variable name="map_index" select="@index"/>
    <xsl:variable name="map_width" select="@width"/>
    <xsl:variable name="map_height" select="@height"/>
    <xsl:variable name="map_offsetx" select="@offsetx"/>
    <xsl:variable name="map_offsety" select="@offsety"/>
    
    <xsl:variable name="file" select="concat($output_path, '/map_', $map_index+1, '.html')"/> 
    <redirect:write file="{$file}">
    <html>
      <head>
        <title>Dungeon: map <xsl:value-of select="$map_index+1"/></title>
        <link rel="stylesheet" type="text/css">
          <xsl:attribute name="href"><xsl:value-of select="$resources_path"/>/styles/dungeon.css</xsl:attribute>
        </link>
        <script type="text/javascript">
          <xsl:attribute name="src"><xsl:value-of select="$resources_path"/>/js/wz_jsgraphics.js</xsl:attribute>
        </script>
        <script type="text/javascript">
          var tpTitle = "Tile description";
          function addContent(content) {
            <xsl:text><![CDATA[if (content.length>0) return ("<div class='tpContent'>"+content+"</div>");]]></xsl:text>
            return "";
          }
          function tpTileCommon(tp, class_id, secondary_class_id, type_name) {
            <xsl:text><![CDATA[return '<span class='+class_id+' style="font-size:]]></xsl:text><xsl:value-of select="$icon_height"/><xsl:text><![CDATA[px">'+((secondary_class_id.length > 0)?'<span class='+secondary_class_id+'></span>':'')+']]></xsl:text><img><xsl:attribute name="src"><xsl:value-of select="$resources_path"/>/images/hidden.png</xsl:attribute></img><xsl:text><![CDATA[</span> Type: '+type_name;]]></xsl:text>
            
          }
          function tpTileGeneric(tp, class_id, x, y, secondary_class_id, type_name) {
            <xsl:text><![CDATA[tp.T_TITLE=tpTitle+" (X="+x+",Y="+y+")";]]></xsl:text>
            return tpTileCommon(tp, class_id, secondary_class_id, type_name);
          }
  
          <xsl:for-each select="tiles/tile[(position()-1) mod $map_width=0]">
            <xsl:variable name="current_line_index" select="position()-1"/>
            <xsl:call-template name="tile-description">
              <xsl:with-param name="map_index"   select="$map_index"/>
              <xsl:with-param name="map_width"   select="$map_width"/>
              <xsl:with-param name="map_height"  select="$map_height"/>
              <xsl:with-param name="coord_y"     select="$current_line_index"/>
              <xsl:with-param name="coord_x"     select="0"/>
            </xsl:call-template>
            <xsl:for-each select="following-sibling::tile[position() &lt; $map_width]">
              <xsl:call-template name="tile-description">
                <xsl:with-param name="map_index"   select="$map_index"/>
                <xsl:with-param name="map_width"   select="$map_width"/>
                <xsl:with-param name="map_height"  select="$map_height"/>
                <xsl:with-param name="coord_y"     select="$current_line_index"/>
              </xsl:call-template>
            </xsl:for-each>
          </xsl:for-each>
        </script>
      </head>
      <body>
      
      <center>
      <table cellpadding="0" class="legend">
        <tr class="legend_title"><td colspan="8">Legend</td></tr>
        <tr>
            <td class="tile_wall"></td><td>Wall</td>
            <td class="tile_wall_trick"></td><td>Trick Wall</td>
            <td class="tile_wall_illusion"></td><td>False Wall</td>
            <td class="tile_floor"></td><td>Floor</td>
        </tr><tr>
            <td class="tile_pit"></td><td>Pit</td>
            <td class="tile_pit_hidden"></td><td>Hidden Pit</td>
            <td class="tile_wall"><span class="tile_teleporter"></span></td><td>Teleporter</td>
            <td class="tile_wall"><span class="tile_teleporter_hidden"></span></td><td>Hidden Teleporter</td>
        </tr><tr>
            <td class="tile_door_0_v"></td><td>Door</td>
            <td class="tile_door_1_v"></td><td>Door</td>
            <td class="tile_door_2_v"></td><td>Door</td>
            <td class="tile_door_3_v"></td><td>Door</td>
        </tr><tr>
            <td class="tile_stairs_up_east"></td><td>Stairs</td>
            <td class="tile_wall"><span class="item_text_west"></span></td><td>Text item</td>
            <td class="tile_floor"><span class="party_west"></span></td><td>Party</td>
            <td class="tile_wall"><span class="wall_type_1_west"></span></td><td>Wall actuator</td>
        </tr><tr>
            <td class="tile_wall"><span class="wall_graphic_1_west"></span></td><td>Alcove</td>
            <td class="tile_wall"><span class="wall_graphic_2_west"></span></td><td>VI altar</td>
            <td class="tile_wall"><span class="wall_graphic_5_west"></span></td><td>Lock</td>
            <td class="tile_wall"><span class="wall_graphic_7_west"></span></td><td>Switch</td>
        </tr><tr>
            <td class="tile_wall"><span class="wall_graphic_35_west"></span></td><td>Fountain</td>
            <td class="tile_wall"><span class="wall_graphic_36_west"></span></td><td>Manacle</td>
            <td class="tile_wall"><span class="wall_graphic_37_west"></span></td><td>Ghoul's head</td>
            <td class="tile_wall"><span class="wall_graphic_38_west"></span></td><td>Torch</td>
        </tr><tr>
            <td class="tile_wall"><span class="wall_graphic_40_west"></span></td><td>Poison missile</td>
            <td class="tile_wall"><span class="wall_graphic_41_west"></span></td><td>Magic missile</td>
            <td class="tile_wall"><span class="wall_graphic_42_west"></span></td><td>Dagger missile</td>
            <td class="tile_wall"><span class="wall_graphic_43_west"></span></td><td>Champion mirror</td>
        </tr><tr>
            <td class="tile_wall"><span class="wall_graphic_56_west"></span></td><td>Amalgam</td>
            <td class="tile_wall"><span class="wall_graphic_59_west"></span></td><td>Lord</td>
            <td class="tile_floor"><span class="floor_graphic_1"></span></td><td>Square pressure pad</td>
            <td class="tile_floor"><span class="floor_graphic_4"></span></td><td>Round pressure pad</td>
        </tr><tr>
            <td class="tile_floor"><span class="floor_graphic_5"></span></td><td>Flame</td>
            <td class="tile_floor"><span class="floor_graphic_7"></span></td><td>Tiny pressure pad</td>
            <td class="tile_floor"><span class="floor_type_1"></span></td><td>Floor actuator</td>
            <td class="tile_floor"><span class="floor_type_6"></span></td><td>Monster generator</td>
        </tr><tr>
            <td class="tile_floor"><span class="item_west"></span></td><td>Item</td>
            <td class="tile_floor"><span class="creature_west"></span></td><td>Creature</td>
            <td><img><xsl:attribute name="src"><xsl:value-of select="$resources_path"/>/images/potion.png</xsl:attribute></img></td><td>Potion (tooltip)</td>
            <td><img><xsl:attribute name="src"><xsl:value-of select="$resources_path"/>/images/creature.png</xsl:attribute></img></td><td>Creature (tooltip)</td>
        </tr><tr>
            <td><img><xsl:attribute name="src"><xsl:value-of select="$resources_path"/>/images/champion.png</xsl:attribute></img></td><td>Champion (tooltip)</td>
            <td><img><xsl:attribute name="src"><xsl:value-of select="$resources_path"/>/images/text.png</xsl:attribute></img></td><td>Text (tooltip)</td>
            <td><img><xsl:attribute name="src"><xsl:value-of select="$resources_path"/>/images/misc.png</xsl:attribute></img></td><td>Miscellaneous (tooltip)</td>
            <td><img><xsl:attribute name="src"><xsl:value-of select="$resources_path"/>/images/container.png</xsl:attribute></img></td><td>Container (tooltip)</td>
        </tr><tr>
            <td><img><xsl:attribute name="src"><xsl:value-of select="$resources_path"/>/images/cloth.png</xsl:attribute></img></td><td>Cloth (tooltip)</td>
            <td><img><xsl:attribute name="src"><xsl:value-of select="$resources_path"/>/images/weapon.png</xsl:attribute></img></td><td>Weapon (tooltip)</td>
            <td><img><xsl:attribute name="src"><xsl:value-of select="$resources_path"/>/images/scroll.png</xsl:attribute></img></td><td>Scroll (tooltip)</td>
        </tr>
      </table>
      </center>
      <br/>
      
      <center>
      <table border="0" cellspacing="0" cellpadding="0" class="map">
        <tr><td class="map_title">Dungeon:&#160;map <xsl:value-of select="$map_index+1"/></td></tr>  
        <tr><td>

      <div id="myCanvas" style="position:relative;">
        <table border="0" cellspacing="0" cellpadding="0">  
        
          <!-- Draw first line: absolute coordinates X -->
          <xsl:if test="$map_offsetx > 0">
            <tr>
              <xsl:if test="$map_offsety > 0"><td class="offset">&#160;</td></xsl:if>
              <td class="offset">&#160;</td>
              <xsl:call-template name="draw_offsetcoordy">
                <xsl:with-param name="count"   select="$map_width+$map_offsetx"/>
                <xsl:with-param name="max"     select="$map_width+$map_offsetx"/>
              </xsl:call-template>
              <!-- draw one additional tile for border random wall decoration -->
              <td class="offset_rel">&#160;</td>
            </tr>
          </xsl:if>
          
          <!-- Draw second line: relatives coordinates X -->
          <tr>
            <xsl:if test="$map_offsety > 0">
              <td class="offset">&#160;</td>
            </xsl:if>
            <td class="offset_rel">&#160;</td>
            <xsl:call-template name="draw_coordy">
              <xsl:with-param name="map_index" select="$map_index"/>
              <xsl:with-param name="count"     select="$map_width + $map_offsetx"/>
              <xsl:with-param name="offsetx"   select="$map_offsetx"/>
              <xsl:with-param name="max"       select="$map_width + $map_offsetx"/>
              <xsl:with-param name="with_rwd"><xsl:if test="$map_offsety &lt;= 0">1</xsl:if><xsl:if test="$map_offsety > 0">0</xsl:if></xsl:with-param>
            </xsl:call-template>
            <!-- draw one additional tile for border random wall decoration -->
            <td class="offset_rel">&#160;</td>
          </tr>
          
          <!-- Draw: tiles offset lines -->
          <xsl:if test="$map_offsety > 0">
            <xsl:call-template name="draw_offsety">
              <xsl:with-param name="map_index" select="$map_index"/>
              <xsl:with-param name="count"     select="$map_offsety"/>
              <xsl:with-param name="width"     select="$map_width + $map_offsetx"/>
              <xsl:with-param name="offsety"   select="$map_offsety"/>
              <xsl:with-param name="with_rwd"  select="1"/>
            </xsl:call-template>
          </xsl:if>
          
          <!-- Draw: tiles -->
          <xsl:for-each select="tiles/tile[(position()-1) mod $map_width=0]">
            <xsl:variable name="current_line_index" select="position()-1"/>
            <tr>
              <xsl:if test="$map_offsety > 0">
                <td class="offset"><xsl:value-of select="position()-1+$map_offsety"/></td>
              </xsl:if>
              <td class="offset_rel">
                <xsl:value-of select="position()-1"/>
                <xsl:if test="$map_offsetx &lt;= 0">
                  <xsl:call-template name="random-decoration">
                    <xsl:with-param name="map_index" select="$map_index"/>
                    <xsl:with-param name="x"         select="-1"/>
                    <xsl:with-param name="y"         select="position()-1"/>
                  </xsl:call-template>
                </xsl:if>
              </td>
              <xsl:if test="$map_offsetx > 0">
                <xsl:call-template name="draw_line">
                  <xsl:with-param name="map_index" select="$map_index"/>
                  <xsl:with-param name="y"         select="position()-1"/>
                  <xsl:with-param name="count"     select="$map_offsetx"/>
                  <xsl:with-param name="width"     select="$map_offsetx"/>
                  <xsl:with-param name="with_rwd"  select="2"/>
                </xsl:call-template>
              </xsl:if>
              <xsl:call-template name="tile">
                <xsl:with-param name="map_index" select="$map_index"/>
                <xsl:with-param name="map_width" select="$map_width"/>
                <xsl:with-param name="map_height" select="$map_height"/>
                <xsl:with-param name="coord_y"   select="$current_line_index"/>
                <xsl:with-param name="coord_x"   select="0"/>
              </xsl:call-template>
              <xsl:for-each select="following-sibling::tile[position() &lt; $map_width]">
                <xsl:call-template name="tile">
                  <xsl:with-param name="map_index" select="$map_index"/>
                  <xsl:with-param name="map_width" select="$map_width"/>
                  <xsl:with-param name="map_height" select="$map_height"/>
                  <xsl:with-param name="coord_y"   select="$current_line_index"/>
                </xsl:call-template>
              </xsl:for-each>
              <!-- draw one additional tile for border random wall decoration -->
              <td class="offset_rel">&#160;
                <xsl:call-template name="random-decoration">
                  <xsl:with-param name="map_index" select="$map_index"/>
                  <xsl:with-param name="x"         select="$map_width"/>
                  <xsl:with-param name="y"         select="position()-1"/>
                </xsl:call-template>
              </td>
            </tr>
          </xsl:for-each>
          
          <!-- Draw last line for random wall decorations -->
          <tr>
            <xsl:if test="$map_offsety > 0">
              <td class="offset_rel">&#160;</td>
            </xsl:if>
            <td class="offset_rel">&#160;</td>
            <xsl:call-template name="random-decorations">
              <xsl:with-param name="map_index" select="$map_index"/>
              <xsl:with-param name="x"         select="0 - $map_offsetx"/>
              <xsl:with-param name="y"         select="$map_height"/>
              <xsl:with-param name="count"     select="$map_width + $map_offsetx"/>
            </xsl:call-template>
            <!-- draw one additional tile for border random wall decoration -->
            <td class="offset_rel">&#160;</td>
          </tr>
          
        </table>
      </div>
      
      </td></tr>
      </table>
      <br/>
      <![CDATA[To print the map, you must turn on the 'print background image' of your browser.]]><br/>
      <![CDATA[[Firefox:File:Page Setup:Options: Print Background (colours & images)]]]><br/>
      <![CDATA[[IE:Tools:Internet Options:Advanced:Printing:Print background colors and images]]]>
      </center>
      
      <!-- Write: tiles content -->
      <br/>
      <table border="1" cellspacing="0" class="tiles_content">
      <tr><td colspan="2" class="tiles_content_title">Tiles content</td></tr>
      <tr class="tiles_content_title"><td>Coords (X;Y)</td><td>Description</td></tr>
      <xsl:for-each select="tiles/tile[(position()-1) mod $map_width=0]">
        <xsl:variable name="current_line_index" select="position()-1"/>
        <div>
          <xsl:call-template name="tile-content">
            <xsl:with-param name="map_index"  select="$map_index"/>
            <xsl:with-param name="map_width"  select="$map_width"/>
            <xsl:with-param name="map_height" select="$map_height"/>
            <xsl:with-param name="coord_y"    select="$current_line_index"/>
            <xsl:with-param name="coord_x"    select="0"/>
          </xsl:call-template>
          <xsl:for-each select="following-sibling::tile[position() &lt; $map_width]">
            <xsl:call-template name="tile-content">
              <xsl:with-param name="map_index"  select="$map_index"/>
              <xsl:with-param name="map_width"  select="$map_width"/>
              <xsl:with-param name="map_height" select="$map_height"/>
              <xsl:with-param name="coord_y"    select="$current_line_index"/>
            </xsl:call-template>
           </xsl:for-each>
        </div>
      </xsl:for-each>
      </table>
      
      <!-- Draw: actuators/teleporters lines -->
      <script type="text/javascript">
      function myDrawFunction(){
        jg.setZIndex("100");
        <xsl:for-each select="tiles/tile[(position()-1) mod $map_width=0]">
          <xsl:variable name="current_line_index" select="position()-1"/>
          <xsl:if test="count(items/actuator)!=0">
            <xsl:call-template name="actuator-line">
              <xsl:with-param name="map_index"   select="$map_index"/>
              <xsl:with-param name="map_width"   select="$map_width"/>
              <xsl:with-param name="map_height"  select="$map_height"/>
              <xsl:with-param name="map_offsetx" select="$map_offsetx"/>
              <xsl:with-param name="map_offsety" select="$map_offsety"/>
              <xsl:with-param name="coord_y"     select="$current_line_index"/>
              <xsl:with-param name="coord_x"     select="0"/>
            </xsl:call-template>
          </xsl:if>
          <xsl:for-each select="following-sibling::tile[position() &lt; $map_width]">
            <xsl:if test="count(items/actuator)!=0">
              <xsl:call-template name="actuator-line">
                <xsl:with-param name="map_index"   select="$map_index"/>
                <xsl:with-param name="map_width"   select="$map_width"/>
                <xsl:with-param name="map_height"  select="$map_height"/>
                <xsl:with-param name="map_offsetx" select="$map_offsetx"/>
                <xsl:with-param name="map_offsety" select="$map_offsety"/>
                <xsl:with-param name="coord_y"     select="$current_line_index"/>
              </xsl:call-template>
            </xsl:if>
          </xsl:for-each>
          <xsl:if test="count(items/teleporter)!=0">
            <xsl:call-template name="teleporter-line">
              <xsl:with-param name="map_index"   select="$map_index"/>
              <xsl:with-param name="map_width"   select="$map_width"/>
              <xsl:with-param name="map_height"  select="$map_height"/>
              <xsl:with-param name="map_offsetx" select="$map_offsetx"/>
              <xsl:with-param name="map_offsety" select="$map_offsety"/>
              <xsl:with-param name="coord_y"     select="$current_line_index"/>
              <xsl:with-param name="coord_x"     select="0"/>
            </xsl:call-template>
          </xsl:if>
          <xsl:for-each select="following-sibling::tile[position() &lt; $map_width]">
            <xsl:if test="count(items/teleporter)!=0">
              <xsl:call-template name="teleporter-line">
                <xsl:with-param name="map_index"   select="$map_index"/>
                <xsl:with-param name="map_width"   select="$map_width"/>
                <xsl:with-param name="map_height"  select="$map_height"/>
                <xsl:with-param name="map_offsetx" select="$map_offsetx"/>
                <xsl:with-param name="map_offsety" select="$map_offsety"/>
                <xsl:with-param name="coord_y"     select="$current_line_index"/>
              </xsl:call-template>
            </xsl:if>
          </xsl:for-each>
        </xsl:for-each>
        jg.paint();
      }
      function myClearFunction(){
        jg.clear();
      }
      function drawVectors(checkbox) {
        if (checkbox.checked) myDrawFunction();
        else myClearFunction();
      }
      var jg = new jsGraphics("myCanvas");
      myDrawFunction();
      </script>
            
      <script type="text/javascript">
        <xsl:attribute name="src"><xsl:value-of select="$resources_path"/>/js/wz_tooltip.js</xsl:attribute>
      </script>
           
      </body>
    </html>
    </redirect:write>
  </xsl:for-each>
</xsl:template>

<!-- Draw: map tile -->
<xsl:template name="tile">
  <xsl:param name="map_index"  select="0"/>
  <xsl:param name="map_width"  select="1"/>
  <xsl:param name="map_height" select="1"/>
  <xsl:param name="coord_y"    select="0"/>
  <xsl:param name="coord_x"    select="position()"/>
  <xsl:variable name="absolute_tile_position" select="$coord_y * $map_width + $coord_x + 1"/>  
  <xsl:variable name="door_index" select="items/door/@index"/>
  <xsl:variable name="class_id">
    <xsl:if test="@type=0">tile_wall</xsl:if>
    <xsl:if test="@type=1">tile_floor</xsl:if>
    <xsl:if test="@type=2">
      <xsl:if test="@is_invisible=1">tile_pit_hidden</xsl:if>
      <xsl:if test="@is_invisible=0">tile_pit</xsl:if>
    </xsl:if>
    <xsl:if test="@type=3">
      <xsl:if test="@orientation=0">
        <xsl:if test="@direction=0">
          <xsl:if test="($coord_x=0) or ((($coord_x + 1) != $map_width) and (number(/dungeon/maps/map[position()=($map_index+1)]/tiles/tile[position()=($absolute_tile_position + 1)]/@type)!=0))"
          >tile_stairs_down_west</xsl:if>
          <xsl:if test="(($coord_x + 1)=$map_width) or (($coord_x != 0) and (number(/dungeon/maps/map[position()=($map_index+1)]/tiles/tile[position()=($absolute_tile_position - 1)]/@type)!=0))"
          >tile_stairs_down_east</xsl:if>
        </xsl:if>
        <xsl:if test="@direction=1">
          <xsl:if test="($coord_x=0) or ((($coord_x + 1) != $map_width) and (number(/dungeon/maps/map[position()=($map_index+1)]/tiles/tile[position()=($absolute_tile_position + 1)]/@type)!=0))"
          >tile_stairs_up_west</xsl:if>
          <xsl:if test="(($coord_x + 1)=$map_width) or (($coord_x != 0) and (number(/dungeon/maps/map[position()=($map_index+1)]/tiles/tile[position()=($absolute_tile_position - 1)]/@type)!=0))"
          >tile_stairs_up_east</xsl:if>
        </xsl:if>
      </xsl:if>
      <xsl:if test="@orientation=1">
        <xsl:if test="@direction=0">
          <xsl:if test="($coord_y=0) or ((($coord_y + 1) != $map_height) and (number(/dungeon/maps/map[position()=($map_index+1)]/tiles/tile[position()=($absolute_tile_position + $map_width)]/@type)!=0))"
          >tile_stairs_down_north</xsl:if>
          <xsl:if test="(($coord_y + 1)=$map_height) or (($coord_y != 0) and (number(/dungeon/maps/map[position()=($map_index+1)]/tiles/tile[position()=($absolute_tile_position - $map_width)]/@type)!=0))"
          >tile_stairs_down_south</xsl:if>
        </xsl:if>
        <xsl:if test="@direction=1">
          <xsl:if test="($coord_y=0) or ((($coord_y + 1) != $map_height) and (number(/dungeon/maps/map[position()=($map_index+1)]/tiles/tile[position()=($absolute_tile_position + $map_width)]/@type)!=0))"
          >tile_stairs_up_north</xsl:if>
          <xsl:if test="(($coord_y + 1)=$map_height) or (($coord_y != 0) and (number(/dungeon/maps/map[position()=($map_index+1)]/tiles/tile[position()=($absolute_tile_position - $map_width)]/@type)!=0))"
          >tile_stairs_up_south</xsl:if>
        </xsl:if>
      </xsl:if>
    </xsl:if>
    <xsl:if test="@type=4">
      <xsl:text>tile_door_</xsl:text>
      <xsl:if test="/dungeon/objects/doors/door[@index=$door_index]/@type='0'">
        <xsl:text><xsl:value-of select="/dungeon/maps/map[position()=($map_index+1)]/@graphics_door0"/></xsl:text>
      </xsl:if>
      <xsl:if test="/dungeon/objects/doors/door[@index=$door_index]/@type='1'">
        <xsl:text><xsl:value-of select="/dungeon/maps/map[position()=($map_index+1)]/@graphics_door1"/></xsl:text>
      </xsl:if>
      <xsl:if test="@orientation=1"><xsl:text>_h</xsl:text></xsl:if>
      <xsl:if test="@orientation=0"><xsl:text>_v</xsl:text></xsl:if>
    </xsl:if>
    <!-- Draw teleporter as a floor -->
    <xsl:if test="@type=5">tile_floor</xsl:if>
    <xsl:if test="@type=6">
      <xsl:if test="@is_imaginary=1">tile_wall_trick</xsl:if>
      <xsl:if test="@is_imaginary=0">tile_wall_illusion</xsl:if>
    </xsl:if>
    <xsl:if test="@type>6">tile_unknown</xsl:if>
  </xsl:variable>
  <xsl:variable name="secondary_class_id">
    <xsl:if test="@type=5">
      <xsl:if test="@is_visible=1">tile_teleporter</xsl:if>
      <xsl:if test="@is_visible=0">tile_teleporter_hidden</xsl:if>
    </xsl:if>
  </xsl:variable>
  <xsl:variable name="tile_content"><xsl:call-template name="tile-content-tp"></xsl:call-template></xsl:variable>
    
  <td>
    <xsl:attribute name="class"><xsl:value-of select="$class_id"/></xsl:attribute>
    <xsl:attribute name="onmouseover"><xsl:if test="string-length($tile_content) > 0">return escape(tpTile_<xsl:value-of select="$coord_x"/>_<xsl:value-of select="$coord_y"/>(this, '<xsl:value-of select="$class_id"/>'))</xsl:if>
                                      <xsl:if test="string-length($tile_content) &lt;= 0">return escape(tpTileGeneric(this, '<xsl:value-of select="$class_id"/>',<xsl:value-of select="$coord_x"/>,<xsl:value-of select="$coord_y"/>,'<xsl:value-of select="$secondary_class_id"/>','<xsl:call-template name="tile-type-fullname"><xsl:with-param name="map_index" select="$map_index"/></xsl:call-template>'))</xsl:if>
    </xsl:attribute>
    
    <!-- Draw teleporter image on teleporter floor -->
    <xsl:if test="@type=5">
      <span>
        <xsl:attribute name="class">
          <xsl:if test="@is_visible=1">tile_teleporter</xsl:if>
          <xsl:if test="@is_visible=0">tile_teleporter_hidden</xsl:if>
        </xsl:attribute>
      </span>
    </xsl:if>
    
    <!-- Wall tile: Draw random decorations -->
    <xsl:if test="@type=0 or @type=6">
      <xsl:if test="count(items/actuator[@position='0'])=0">
        <xsl:for-each select="items/random_wall_decoration[@position='0']">
          <xsl:variable name="decoration_graphic"      select="number(@graphic) - 1"/>
          <xsl:variable name="decoration_graphic_type" select="number(/dungeon/maps/map[position()=($map_index+1)]/wall_decoration_graphics/wall_decoration_graphic[@index=$decoration_graphic]/@type)"/>
          <xsl:if test="$decoration_graphic!=-1">
            <span>
              <xsl:attribute name="class">
                <xsl:text>wall_graphic_</xsl:text><xsl:value-of select="$decoration_graphic_type"/><xsl:text>_north</xsl:text>
              </xsl:attribute>
            </span>
          </xsl:if>
        </xsl:for-each>
      </xsl:if>
      <xsl:if test="count(items/actuator[@position='1'])=0">
        <xsl:for-each select="items/random_wall_decoration[@position='1']">
          <xsl:variable name="decoration_graphic"      select="number(@graphic) - 1"/>
          <xsl:variable name="decoration_graphic_type" select="number(/dungeon/maps/map[position()=($map_index+1)]/wall_decoration_graphics/wall_decoration_graphic[@index=$decoration_graphic]/@type)"/>
          <xsl:if test="$decoration_graphic!=-1">
            <span>
              <xsl:attribute name="class">
                <xsl:text>wall_graphic_</xsl:text><xsl:value-of select="$decoration_graphic_type"/><xsl:text>_east</xsl:text>
              </xsl:attribute>
            </span>
          </xsl:if>
        </xsl:for-each>
      </xsl:if>
      <xsl:if test="count(items/actuator[@position='2'])=0">
        <xsl:for-each select="items/random_wall_decoration[@position='2']">
          <xsl:variable name="decoration_graphic"      select="number(@graphic) - 1"/>
          <xsl:variable name="decoration_graphic_type" select="number(/dungeon/maps/map[position()=($map_index+1)]/wall_decoration_graphics/wall_decoration_graphic[@index=$decoration_graphic]/@type)"/>
          <xsl:if test="$decoration_graphic!=-1">
            <span>
              <xsl:attribute name="class">
                <xsl:text>wall_graphic_</xsl:text><xsl:value-of select="$decoration_graphic_type"/><xsl:text>_south</xsl:text>
              </xsl:attribute>
            </span>
          </xsl:if>
        </xsl:for-each>
      </xsl:if>
      <xsl:if test="count(items/actuator[@position='3'])=0">
        <xsl:for-each select="items/random_wall_decoration[@position='3']">
          <xsl:variable name="decoration_graphic"      select="number(@graphic) - 1"/>
          <xsl:variable name="decoration_graphic_type" select="number(/dungeon/maps/map[position()=($map_index+1)]/wall_decoration_graphics/wall_decoration_graphic[@index=$decoration_graphic]/@type)"/>
          <xsl:if test="$decoration_graphic!=-1">
            <span>
              <xsl:attribute name="class">
                <xsl:text>wall_graphic_</xsl:text><xsl:value-of select="$decoration_graphic_type"/><xsl:text>_west</xsl:text>
              </xsl:attribute>
            </span>
          </xsl:if>
        </xsl:for-each>
      </xsl:if>
    </xsl:if>
    
    <!-- Wall tile: Draw non-random decorations -->
    <xsl:if test="@type=0">
    
      <xsl:for-each select="items/actuator[@position='0']">
        <xsl:sort order="descending"/>
        <xsl:variable name="actuator_type"         select="number(type)"/>
        <xsl:variable name="actuator_graphic"      select="number(graphic) - 1"/>
        <xsl:variable name="actuator_graphic_type" select="number(/dungeon/maps/map[position()=($map_index+1)]/wall_decoration_graphics/wall_decoration_graphic[@index=$actuator_graphic]/@type)"/>
        <span>
          <xsl:attribute name="class">
            <xsl:text>wall_</xsl:text>
            <xsl:if test="not(position()=last())">
              <xsl:text>type_</xsl:text><xsl:value-of select="$actuator_type"/>
            </xsl:if>
            <xsl:if test="position()=last()">
              <xsl:if test="$actuator_graphic!=-1">
                <xsl:text>graphic_</xsl:text><xsl:value-of select="$actuator_graphic_type"/>
              </xsl:if>
              <xsl:if test="$actuator_graphic=-1">
                <xsl:text>type_</xsl:text><xsl:value-of select="$actuator_type"/>
              </xsl:if>
            </xsl:if>
            <xsl:text>_north</xsl:text>
            </xsl:attribute>
        </span>
      </xsl:for-each>
      
      <xsl:for-each select="items/actuator[@position='1']">
        <xsl:sort order="descending"/>
        <xsl:variable name="actuator_type"         select="number(type)"/>
        <xsl:variable name="actuator_graphic"      select="number(graphic) - 1"/>
        <xsl:variable name="actuator_graphic_type" select="number(/dungeon/maps/map[position()=($map_index+1)]/wall_decoration_graphics/wall_decoration_graphic[@index=$actuator_graphic]/@type)"/>
        <span>
          <xsl:attribute name="class">
            <xsl:text>wall_</xsl:text>
            <xsl:if test="not(position()=last())">
              <xsl:text>type_</xsl:text><xsl:value-of select="$actuator_type"/>
            </xsl:if>
            <xsl:if test="position()=last()">
              <xsl:if test="$actuator_graphic!=-1">
                <xsl:text>graphic_</xsl:text><xsl:value-of select="$actuator_graphic_type"/>
              </xsl:if>
              <xsl:if test="$actuator_graphic=-1">
                <xsl:text>type_</xsl:text><xsl:value-of select="$actuator_type"/>
              </xsl:if>
            </xsl:if>
            <xsl:text>_east</xsl:text>
            </xsl:attribute>
        </span>
      </xsl:for-each>
      
      <xsl:for-each select="items/actuator[@position='2']">
        <xsl:sort order="descending"/>
        <xsl:variable name="actuator_type"         select="number(type)"/>
        <xsl:variable name="actuator_graphic"      select="number(graphic) - 1"/>
        <xsl:variable name="actuator_graphic_type" select="number(/dungeon/maps/map[position()=($map_index+1)]/wall_decoration_graphics/wall_decoration_graphic[@index=$actuator_graphic]/@type)"/>
        <span>
          <xsl:attribute name="class">
            <xsl:text>wall_</xsl:text>
            <xsl:if test="not(position()=last())">
              <xsl:text>type_</xsl:text><xsl:value-of select="$actuator_type"/>
            </xsl:if>
            <xsl:if test="position()=last()">
              <xsl:if test="$actuator_graphic!=-1">
                <xsl:text>graphic_</xsl:text><xsl:value-of select="$actuator_graphic_type"/>
              </xsl:if>
              <xsl:if test="$actuator_graphic=-1">
                <xsl:text>type_</xsl:text><xsl:value-of select="$actuator_type"/>
              </xsl:if>
            </xsl:if>
            <xsl:text>_south</xsl:text>
            </xsl:attribute>
        </span>
      </xsl:for-each>
      
      <xsl:for-each select="items/actuator[@position='3']">
        <xsl:sort order="descending"/>
        <xsl:variable name="actuator_type"         select="number(type)"/>
        <xsl:variable name="actuator_graphic"      select="number(graphic) - 1"/>
        <xsl:variable name="actuator_graphic_type" select="number(/dungeon/maps/map[position()=($map_index+1)]/wall_decoration_graphics/wall_decoration_graphic[@index=$actuator_graphic]/@type)"/>
        <span>
          <xsl:attribute name="class">
            <xsl:text>wall_</xsl:text>
            <xsl:if test="not(position()=last())">
              <xsl:text>type_</xsl:text><xsl:value-of select="$actuator_type"/>
            </xsl:if>
            <xsl:if test="position()=last()">
              <xsl:if test="$actuator_graphic!=-1">
                <xsl:text>graphic_</xsl:text><xsl:value-of select="$actuator_graphic_type"/>
              </xsl:if>
              <xsl:if test="$actuator_graphic=-1">
                <xsl:text>type_</xsl:text><xsl:value-of select="$actuator_type"/>
              </xsl:if>
            </xsl:if>
            <xsl:text>_west</xsl:text>
            </xsl:attribute>
        </span>
      </xsl:for-each>
      
    </xsl:if>
    
    <!-- Wall tile: Draw non-random text item -->
    <xsl:if test="@type=0">
      <xsl:variable name="text_index0" select="number(items/text[@position=0][last()]/@index)"/>
      <xsl:if test="count(/dungeon/objects/texts/text[@visible=1 and @index=$text_index0])&gt;0">
        <span>
          <xsl:attribute name="class"><xsl:text>item_text_north</xsl:text></xsl:attribute>
        </span>
      </xsl:if>
      <xsl:variable name="text_index1" select="number(items/text[@position=1][last()]/@index)"/>
      <xsl:if test="count(/dungeon/objects/texts/text[@visible=1 and @index=$text_index1])&gt;0">
        <span>
          <xsl:attribute name="class"><xsl:text>item_text_east</xsl:text></xsl:attribute>
        </span>
      </xsl:if>
      <xsl:variable name="text_index2" select="number(items/text[@position=2][last()]/@index)"/>
      <xsl:if test="count(/dungeon/objects/texts/text[@visible=1 and @index=$text_index2])&gt;0">
        <span>
          <xsl:attribute name="class"><xsl:text>item_text_south</xsl:text></xsl:attribute>
        </span>
      </xsl:if>
      <xsl:variable name="text_index3" select="number(items/text[@position=3][last()]/@index)"/>
      <xsl:if test="count(/dungeon/objects/texts/text[@visible=1 and @index=$text_index3])&gt;0">
        <span>
          <xsl:attribute name="class"><xsl:text>item_text_west</xsl:text></xsl:attribute>
        </span>
      </xsl:if>
    </xsl:if>
    
    <!-- Floor tile: Draw non-random decoration and actuator -->
    <xsl:if test="@type=1 or @type=2 or @type=3 or @type=4 or @type=5">
      <xsl:for-each select="items/actuator">
        <xsl:variable name="actuator_type"         select="number(type)"/>
        <xsl:variable name="actuator_graphic"      select="number(graphic) - 1"/>
        <xsl:variable name="actuator_graphic_type" select="number(/dungeon/maps/map[position()=($map_index+1)]/floor_decoration_graphics/floor_decoration_graphic[@index=$actuator_graphic]/@type)"/>
        <span>          
          <xsl:attribute name="class">
            <xsl:text>floor_</xsl:text>
            <xsl:if test="not(position()=last())">
              <xsl:text>type_</xsl:text><xsl:value-of select="$actuator_type"/>
            </xsl:if>
            <xsl:if test="position()=last()">
              <xsl:if test="$actuator_graphic!=-1">
                <xsl:text>graphic_</xsl:text><xsl:value-of select="$actuator_graphic_type"/>
              </xsl:if>
              <xsl:if test="$actuator_graphic=-1">
                <xsl:text>type_</xsl:text><xsl:value-of select="$actuator_type"/>
              </xsl:if>
            </xsl:if>
            </xsl:attribute>
        </span>
      </xsl:for-each>
    </xsl:if>
    
    <!-- Floor tile: Draw random decoration -->
    <xsl:if test="@type=1">
      <xsl:if test="count(items/actuator)=0">
        <xsl:for-each select="items/random_floor_decoration">
          <xsl:variable name="decoration_graphic"      select="number(@graphic) - 1"/>
          <xsl:variable name="decoration_graphic_type" select="number(/dungeon/maps/map[position()=($map_index+1)]/floor_decoration_graphics/floor_decoration_graphic[@index=$decoration_graphic]/@type)"/>
          <xsl:if test="$decoration_graphic!=-1">
            <span>
              <xsl:attribute name="class">
                <xsl:text>floor_graphic_</xsl:text><xsl:value-of select="$decoration_graphic_type"/>
              </xsl:attribute>
            </span>
          </xsl:if>
        </xsl:for-each>
      </xsl:if>
    </xsl:if>
    
    <!-- Floor tile: Draw items -->
    <xsl:if test="@type=0 or @type=1 or @type=2 or @type=3 or @type=4 or @type=5 or @type=6">
      <xsl:if test="count(items/child::*[@position='0' and (local-name()='cloth' or local-name()='container' or local-name()='miscellaneous' or local-name()='missile' or local-name()='potion' or local-name()='scroll' or local-name()='weapon')])&gt;0">
        <span class="item_north"></span>
      </xsl:if>
      <xsl:if test="count(items/child::*[@position='1' and (local-name()='cloth' or local-name()='container' or local-name()='miscellaneous' or local-name()='missile' or local-name()='potion' or local-name()='scroll' or local-name()='weapon')])&gt;0">
        <span class="item_east"></span>
      </xsl:if>
      <xsl:if test="count(items/child::*[@position='2' and (local-name()='cloth' or local-name()='container' or local-name()='miscellaneous' or local-name()='missile' or local-name()='potion' or local-name()='scroll' or local-name()='weapon')])&gt;0">
        <span class="item_south"></span>
      </xsl:if>
      <xsl:if test="count(items/child::*[@position='3' and (local-name()='cloth' or local-name()='container' or local-name()='miscellaneous' or local-name()='missile' or local-name()='potion' or local-name()='scroll' or local-name()='weapon')])&gt;0">
        <span class="item_west"></span>
      </xsl:if>
    </xsl:if>
    
    <!-- Floor tile: Draw monsters -->
    <xsl:if test="@type=1 or @type=2 or @type=3 or @type=4 or @type=5 or @type=6">
      <xsl:if test="count(items/creature[@position='0'])&gt;0">
        <span class="creature_north"></span>
      </xsl:if>
      <xsl:if test="count(items/creature[@position='1'])&gt;0">
        <span class="creature_east"></span>
      </xsl:if>
      <xsl:if test="count(items/creature[@position='2'])&gt;0">
        <span class="creature_south"></span>
      </xsl:if>
      <xsl:if test="count(items/creature[@position='3'])&gt;0">
        <span class="creature_west"></span>
      </xsl:if>
    </xsl:if>
    
    <!-- Draw party mask -->
    <xsl:if test="$coord_x=/dungeon/@start_x and $coord_y=/dungeon/@start_y and $map_index=0">
      <span>
        <xsl:attribute name="class">
          <xsl:text>party</xsl:text>
          <xsl:if test="/dungeon/@start_facing='North'">_north</xsl:if>
          <xsl:if test="/dungeon/@start_facing='East'" >_east</xsl:if>
          <xsl:if test="/dungeon/@start_facing='South'">_south</xsl:if>
          <xsl:if test="/dungeon/@start_facing='West'" >_west</xsl:if>
        </xsl:attribute>
      </span>
    </xsl:if>
    
  </td>
  
</xsl:template>

<!-- Draw: offset lines -->
<xsl:template name="draw_offsety">
  <xsl:param name="map_index" select="0"/>
  <xsl:param name="count"     select="1"/>
  <xsl:param name="width"     select="1"/>
  <xsl:param name="offsety"   select="1"/>
  <xsl:param name="with_rwd"  select="0"/>
  <xsl:if test="$count > 0">
    <tr>
      <xsl:if test="$offsety > 0">
        <td class="offset"><xsl:value-of select="$offsety - $count"/></td>
      </xsl:if>
      <td class="offset_rel">&#160;</td>
      <xsl:call-template name="draw_line">
        <xsl:with-param name="map_index" select="$map_index"/>
        <xsl:with-param name="y"         select="-1"/>
        <xsl:with-param name="count"     select="$width"/>
        <xsl:with-param name="width"     select="$width"/>
        <xsl:with-param name="with_rwd"><xsl:if test="$count='1'"><xsl:value-of select="$with_rwd"/></xsl:if><xsl:if test="$count!='1'">0</xsl:if></xsl:with-param>
      </xsl:call-template>
      <!-- draw one additional tile for border random wall decoration -->
      <td class="offset_rel">&#160;</td>
    </tr>
    <xsl:call-template name="draw_offsety">
      <xsl:with-param name="map_index" select="$map_index"/>
      <xsl:with-param name="count"     select="$count - 1"/>
      <xsl:with-param name="width"     select="$width"/>
      <xsl:with-param name="offsety"   select="$offsety"/>
      <xsl:with-param name="with_rwd"  select="$with_rwd"/>
    </xsl:call-template>
  </xsl:if>
</xsl:template>

<!-- Draw: offset tile -->
<xsl:template name="draw_line">
  <xsl:param name="map_index" select="0"/>
  <xsl:param name="y"         select="0"/>
  <xsl:param name="count"     select="1"/>
  <xsl:param name="width"     select="1"/>
  <xsl:param name="with_rwd"  select="0"/>
  <xsl:if test="$count > 0">
    <td class="tile_offset">
      <xsl:if test="$with_rwd=1">
        <xsl:call-template name="random-decoration">
          <xsl:with-param name="map_index" select="$map_index"/>
          <xsl:with-param name="x"         select="$width - $count"/>
          <xsl:with-param name="y"         select="$y"/>
        </xsl:call-template>
      </xsl:if>
      <xsl:if test="$with_rwd=2 and $count=1">
        <xsl:call-template name="random-decoration">
          <xsl:with-param name="map_index" select="$map_index"/>
          <xsl:with-param name="x"         select="-1"/>
          <xsl:with-param name="y"         select="$y"/>
        </xsl:call-template>
      </xsl:if>
    </td>
    <xsl:call-template name="draw_line">
      <xsl:with-param name="map_index" select="$map_index"/>
      <xsl:with-param name="y"         select="$y"/>
      <xsl:with-param name="count"     select="$count - 1"/>
      <xsl:with-param name="width"     select="$width"/>
      <xsl:with-param name="with_rwd"  select="$with_rwd"/>
    </xsl:call-template>
  </xsl:if>
</xsl:template>

<!-- Draw: coordinates y -->
<xsl:template name="draw_coordy">
  <xsl:param name="map_index" select="0"/>
  <xsl:param name="count"     select="1"/>
  <xsl:param name="max"       select="1"/>
  <xsl:param name="offsetx"   select="1"/>
  <xsl:param name="with_rwd"  select="0"/>
  <xsl:if test="$count > 0">
    <xsl:if test="$max - $count &lt;= $offsetx">
      <td class="offset_rel"><xsl:value-of select="$max - $count - $offsetx"/>
        <xsl:if test="$with_rwd='1'">
          <xsl:call-template name="random-decoration">
            <xsl:with-param name="map_index" select="$map_index"/>
            <xsl:with-param name="x"         select="$max - $count - $offsetx"/>
            <xsl:with-param name="y"         select="-1"/>
          </xsl:call-template>
        </xsl:if>
      </td>
    </xsl:if>
    <xsl:if test="$max - $count > $offsetx">
      <td class="offset_rel">&#160;</td>
    </xsl:if>
    <xsl:call-template name="draw_coordy">
      <xsl:with-param name="map_index" select="$map_index"/>
      <xsl:with-param name="count"     select="$count - 1"/>
      <xsl:with-param name="max"       select="$max"/>
      <xsl:with-param name="offsetx"   select="$offsetx"/>
      <xsl:param name="with_rwd"       select="$with_rwd"/>
    </xsl:call-template>
  </xsl:if>
</xsl:template>

<!-- Draw: offset coordinates y -->
<xsl:template name="draw_offsetcoordy">
  <xsl:param name="count"   select="1"/>
  <xsl:param name="max"     select="1"/>
  <xsl:if test="$count > 0">
    <td class="offset"><xsl:value-of select="$max - $count"/></td>
    <xsl:call-template name="draw_offsetcoordy">
      <xsl:with-param name="count"   select="$count - 1"/>
      <xsl:with-param name="max"     select="$max"/>
    </xsl:call-template>
  </xsl:if>
</xsl:template>

<!--the template responsible for changing cases-->
<!--it changes the first letter to upper case and the remaining letters to lower case-->
<xsl:template name="change-case">
  <xsl:param name="item"/>
  <xsl:variable name="first-char" select="translate(substring($item,1,1),'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
  <xsl:variable name="remainder"  select="normalize-space(translate(substring($item,2),'ABCDEFGHIJKLMNOPQRSTUVWXYZ_','abcdefghijklmnopqrstuvwxyz'))"/>
  <xsl:value-of select="concat($first-char,$remainder)"/>
</xsl:template>

<!-- Write: actuators lines -->
<xsl:template name="actuator-line">
  <xsl:param name="map_index"   select="0"/>
  <xsl:param name="map_width"   select="1"/>
  <xsl:param name="map_height"  select="1"/>
  <xsl:param name="map_offsetx" select="0"/>
  <xsl:param name="map_offsety" select="0"/>
  <xsl:param name="coord_y"     select="0"/>
  <xsl:param name="coord_x"     select="position()"/>
  <xsl:for-each select="items/actuator">
    <xsl:if test="action_target='remote' and ($coord_x!=target_x or $coord_y!=target_y)">
      <xsl:variable name="coord_offsetx"><xsl:if test="$map_offsety>0">2</xsl:if><xsl:if test="$map_offsety&lt;=0">1</xsl:if></xsl:variable>
      <xsl:variable name="coord_offsety"><xsl:if test="$map_offsetx>0">2</xsl:if><xsl:if test="$map_offsetx&lt;=0">1</xsl:if></xsl:variable>
      <xsl:variable name="src_x"><xsl:choose>
        <xsl:when test="@position='0' or @position='2'"><xsl:value-of select="($coord_offsetx + $map_offsetx + $coord_x)*$icon_width + $icon_width div 2"/></xsl:when>
        <xsl:when test="@position='1'"><xsl:value-of select="($coord_offsetx + $map_offsetx + $coord_x)*$icon_width + $icon_width div 2 + $icon_width div 4"/></xsl:when>
        <xsl:when test="@position='3'"><xsl:value-of select="($coord_offsetx + $map_offsetx + $coord_x)*$icon_width + $icon_width div 4"/></xsl:when>
      </xsl:choose></xsl:variable>
      <xsl:variable name="src_y"><xsl:choose>
        <xsl:when test="@position='0'"><xsl:value-of select="($coord_offsety + $map_offsety + $coord_y)*$icon_height + $icon_height div 4"/></xsl:when>
        <xsl:when test="@position='1' or @position='3'"><xsl:value-of select="($coord_offsety + $map_offsety + $coord_y)*$icon_height + $icon_height div 2"/></xsl:when>
        <xsl:when test="@position='2'"><xsl:value-of select="($coord_offsety + $map_offsety + $coord_y)*$icon_height + $icon_height div 2 + $icon_height div 4"/></xsl:when>
      </xsl:choose></xsl:variable>
      jg.setColor("<xsl:value-of select="$actuator_line_color"/>");
      jg.drawLine(<xsl:value-of select="$src_x"/>, <xsl:value-of select="$src_y"/>, <xsl:value-of select="($coord_offsetx + $map_offsetx + target_x)*$icon_width + $icon_width div 2"/>, <xsl:value-of select="($coord_offsety + $map_offsety + target_y)*$icon_height + $icon_height div 4"/>);
    </xsl:if>
  </xsl:for-each>
</xsl:template>

<!-- Write: teleporters lines -->
<xsl:template name="teleporter-line">
  <xsl:param name="map_index"   select="0"/>
  <xsl:param name="map_width"   select="1"/>
  <xsl:param name="map_height"  select="1"/>
  <xsl:param name="map_offsetx" select="0"/>
  <xsl:param name="map_offsety" select="0"/>
  <xsl:param name="coord_y"     select="0"/>
  <xsl:param name="coord_x"     select="position()"/>
  <xsl:for-each select="items/teleporter">
    <xsl:variable name="dest_map" select="/dungeon/objects/teleporters/teleporter[@index=current()/@index]/@dest_map"/>
    <xsl:variable name="dest_map_x" select="/dungeon/objects/teleporters/teleporter[@index=current()/@index]/@dest_map_x"/>
    <xsl:variable name="dest_map_y" select="/dungeon/objects/teleporters/teleporter[@index=current()/@index]/@dest_map_y"/>
    <xsl:if test="$map_index=$dest_map and ($coord_x!=$dest_map_x or $coord_y!=$dest_map_y)">
      <xsl:variable name="coord_offsetx"><xsl:if test="$map_offsety>0">2</xsl:if><xsl:if test="$map_offsety&lt;=0">1</xsl:if></xsl:variable>
      <xsl:variable name="coord_offsety"><xsl:if test="$map_offsetx>0">2</xsl:if><xsl:if test="$map_offsetx&lt;=0">1</xsl:if></xsl:variable>
      jg.setColor("<xsl:value-of select="$teleporter_line_color"/>");
      jg.drawLine(<xsl:value-of select="($coord_offsetx + $map_offsetx + $coord_x)*$icon_width + $icon_width div 2"/>, <xsl:value-of select="($coord_offsety + $map_offsety + $coord_y)*$icon_height + $icon_height div 2"/>, <xsl:value-of select="($coord_offsetx + $map_offsetx + $dest_map_x)*$icon_width + $icon_width div 2"/>, <xsl:value-of select="($coord_offsety + $map_offsety + $dest_map_y)*$icon_height + $icon_height div 2"/>);
    </xsl:if>
  </xsl:for-each>
</xsl:template>

<!-- Write: tile description -->
<xsl:template name="tile-description">
  <xsl:param name="map_index"  select="0"/>
  <xsl:param name="map_width"  select="1"/>
  <xsl:param name="map_height" select="1"/>
  <xsl:param name="coord_y"    select="0"/>
  <xsl:param name="coord_x"    select="position()"/>
  <xsl:variable name="type"><xsl:choose>
      <xsl:when test="@type='0'">Wall</xsl:when>
      <xsl:when test="@type='1'">Floor</xsl:when>
      <xsl:when test="@type='2'">Pit</xsl:when>
      <xsl:when test="@type='3'">Stairs</xsl:when>
      <xsl:when test="@type='4'">Door</xsl:when>
      <xsl:when test="@type='5'">Teleporter</xsl:when>
      <xsl:when test="@type='6'">Trick Wall</xsl:when>
      <xsl:otherwise>Unknown</xsl:otherwise>
  </xsl:choose></xsl:variable>
  <xsl:variable name="tile_content"><xsl:call-template name="tile-content-tp"></xsl:call-template></xsl:variable>
  <xsl:if test="string-length($tile_content) > 0">
    function tpTile_<xsl:value-of select="$coord_x"/>_<xsl:value-of select="$coord_y"/>(tp, class_id){
      tp.T_TITLE=tpTitle+" (X=<xsl:value-of select="$coord_x"/>,Y=<xsl:value-of select="$coord_y"/>)";
      <xsl:variable name="secondary_class_id">
        <xsl:if test="@type=5">
          <xsl:if test="@is_visible=1">tile_teleporter</xsl:if>
          <xsl:if test="@is_visible=0">tile_teleporter_hidden</xsl:if>
        </xsl:if>
      </xsl:variable>
      return tpTileCommon(tp, class_id, '<xsl:value-of select="$secondary_class_id"/>', "<xsl:call-template name="tile-type-fullname"><xsl:with-param name="map_index" select="$map_index"/></xsl:call-template>")+"<br/>"+addContent("<xsl:value-of select="$tile_content"/>");
    }
  </xsl:if>
</xsl:template>

<!-- Write: tile content for tooltip -->
<xsl:template name="tile-content-tp">
  <xsl:variable name="tile_type" select="@type"/>
  <xsl:for-each select="items/actuator[type='127']"><xsl:text><![CDATA[<img src=']]></xsl:text><xsl:value-of select="$resources_path"/><xsl:text><![CDATA[/images/champion.png'> ]]></xsl:text><xsl:call-template name="change-case">
      <xsl:with-param name="item" select="$doc_xml/dungeon/champions/champion[@type=current()/data]"/></xsl:call-template><xsl:text><![CDATA[<br>]]></xsl:text></xsl:for-each>
  <xsl:for-each select="items/text"><xsl:text><![CDATA[<img src=']]></xsl:text><xsl:value-of select="$resources_path"/><xsl:text><![CDATA[/images/text.png'> ]]></xsl:text><xsl:call-template name="change-case">
      <xsl:with-param name="item" select="/dungeon/objects/texts/text[@index=current()/@index]"/></xsl:call-template><xsl:text><![CDATA[<br>]]></xsl:text></xsl:for-each>
  <xsl:for-each select="items/cloth">
    <xsl:variable name="cloth_type" select="/dungeon/objects/clothes/cloth[@index=current()/@index]/@type"/>
    <xsl:text><![CDATA[<img src=']]></xsl:text><xsl:value-of select="$resources_path"/><xsl:text><![CDATA[/images/cloth.png'> ]]></xsl:text><xsl:value-of select="$doc_xml/dungeon/items/clothes/cloth[@type=$cloth_type]"></xsl:value-of><xsl:text><![CDATA[<br>]]></xsl:text></xsl:for-each>
  <xsl:for-each select="items/potion">
    <xsl:variable name="potion_type" select="/dungeon/objects/potions/potion[@index=current()/@index]/@type"/>
    <xsl:text><![CDATA[<img src=']]></xsl:text><xsl:value-of select="$resources_path"/><xsl:text><![CDATA[/images/potion.png'> ]]></xsl:text><xsl:value-of select="$doc_xml/dungeon/items/potions/potion[@type=$potion_type]"></xsl:value-of><xsl:text><![CDATA[<br>]]></xsl:text></xsl:for-each>
  <xsl:for-each select="items/weapon">
    <xsl:variable name="weapon_type" select="/dungeon/objects/weapons/weapon[@index=current()/@index]/@type"/>
    <xsl:text><![CDATA[<img src=']]></xsl:text><xsl:value-of select="$resources_path"/><xsl:text><![CDATA[/images/weapon.png'> ]]></xsl:text><xsl:value-of select="$doc_xml/dungeon/items/weapons/weapon[@type=$weapon_type]"></xsl:value-of><xsl:text><![CDATA[<br>]]></xsl:text></xsl:for-each>
  <xsl:for-each select="items/container">
    <xsl:variable name="container_type" select="/dungeon/objects/containers/container[@index=current()/@index]/@type"/>
    <xsl:text><![CDATA[<img src=']]></xsl:text><xsl:value-of select="$resources_path"/><xsl:text><![CDATA[/images/container.png'> ]]></xsl:text><xsl:value-of select="$doc_xml/dungeon/items/containers/container[@type=$container_type]"></xsl:value-of>
    <xsl:text><![CDATA[<br/>"+addContent("]]></xsl:text><xsl:call-template name="tile-content-tp"></xsl:call-template><xsl:text><![CDATA[")+"]]></xsl:text>
    </xsl:for-each>
  <xsl:for-each select="items/scroll"><xsl:text><![CDATA[<img src=']]></xsl:text><xsl:value-of select="$resources_path"/><xsl:text><![CDATA[/images/scroll.png'> (]]></xsl:text><xsl:call-template name="change-case">
      <xsl:with-param name="item" select="/dungeon/objects/texts/text[@index=/dungeon/objects/scrolls/scroll[@index=current()/@index]/@referred_text]/text()"/></xsl:call-template>)<xsl:text><![CDATA[<br>]]></xsl:text></xsl:for-each>
  <xsl:for-each select="items/miscellaneous">
    <xsl:variable name="miscellaneous_type" select="/dungeon/objects/miscellaneouses/miscellaneous[@index=current()/@index]/@type"/>
    <xsl:text><![CDATA[<img src=']]></xsl:text><xsl:value-of select="$resources_path"/><xsl:text><![CDATA[/images/misc.png'> ]]></xsl:text><xsl:value-of select="$doc_xml/dungeon/items/miscellaneouses/miscellaneous[@type=$miscellaneous_type]"></xsl:value-of><xsl:text><![CDATA[<br>]]></xsl:text></xsl:for-each>
  <xsl:for-each select="items/creature">
    <xsl:variable name="creature_type" select="/dungeon/objects/creatures/creature[@index=current()/@index]/@type"/>
    <xsl:text><![CDATA[<img src=']]></xsl:text><xsl:value-of select="$resources_path"/><xsl:text><![CDATA[/images/creature.png'> ]]></xsl:text><xsl:value-of select="$doc_xml/dungeon/items/creatures/creature[@type=$creature_type]"></xsl:value-of><xsl:text><![CDATA[<br>]]></xsl:text></xsl:for-each>
  <xsl:for-each select="items/actuator">(<xsl:if test="$tile_type='0'">wall</xsl:if><xsl:if test="$tile_type!='0'">floor</xsl:if> actuator) <xsl:if test="$tile_type='0'">
      <xsl:choose>
        <xsl:when test="type='0'">Inactive</xsl:when>
        <xsl:when test="type='1'">Any item / Empty hand</xsl:when>
        <xsl:when test="type='2'">Any Item / No empty hand</xsl:when>
        <xsl:when test="type='3'">Specific item [data: <xsl:value-of select="data"/>]</xsl:when>
        <xsl:when test="type='4'">Specific item / Removed [data: <xsl:value-of select="data"/>]</xsl:when>
        <xsl:when test="type='5'">And/Or gate</xsl:when>
        <xsl:when test="type='6'">Counter [data: <xsl:value-of select="data"/>]</xsl:when>
        <xsl:when test="type='7'">Weapon missile shooter [data: <xsl:value-of select="$doc_xml/dungeon/weapon_missiles/weapon_missile[@type=current()/data]"/>]</xsl:when>
        <xsl:when test="type='8'">Magic missile shooter [data: <xsl:value-of select="$doc_xml/dungeon/magic_missiles/magic_missile[@type=current()/data]"/>]</xsl:when>
        <xsl:when test="type='9'">Double weapon missile shooter [data: <xsl:value-of select="$doc_xml/dungeon/weapon_missiles/weapon_missile[@type=current()/data]"/>]</xsl:when>
        <xsl:when test="type='10'">Double magic missile shooter [data: <xsl:value-of select="$doc_xml/dungeon/magic_missiles/magic_missile[@type=current()/data]"/>]</xsl:when>
        <xsl:when test="type='11'">Specific item / Removed / Begin [data: <xsl:value-of select="data"/>]</xsl:when>
        <xsl:when test="type='12'">Item creator [data: <xsl:value-of select="data"/>]</xsl:when>
        <xsl:when test="type='13'">Item storage [data: <xsl:value-of select="data"/>]</xsl:when>
        <xsl:when test="type='14'">Item missile shooter [data: <xsl:value-of select="data"/>]</xsl:when>
        <xsl:when test="type='15'">Double item missile shooter [data: <xsl:value-of select="data"/>]</xsl:when>
        <xsl:when test="type='16'">Item exchanger [data: <xsl:value-of select="data"/>]</xsl:when>
        <xsl:when test="type='17'">Specific item / Removed / Begin / Removed [data: <xsl:value-of select="data"/>]</xsl:when>
        <xsl:when test="type='18'">End game</xsl:when>
        <xsl:when test="type='127'">Champion portrait [<xsl:value-of select="$doc_xml/dungeon/champions/champion[@type=current()/data]"/>]</xsl:when>
      </xsl:choose>
    </xsl:if><xsl:if test="$tile_type!='0'">
      <xsl:choose>
        <xsl:when test="type='0'">Inactive</xsl:when>
        <xsl:when test="type='1'">TPCI</xsl:when>
        <xsl:when test="type='2'">TPC</xsl:when>
        <xsl:when test="type='3'">P</xsl:when>
        <xsl:when test="type='4'">I [data: <xsl:value-of select="data"/>]</xsl:when>
        <xsl:when test="type='5'">Stairs</xsl:when>
        <xsl:when test="type='6'">Creature generator [<xsl:value-of select="number_of_creatures"/><xsl:if test="is_max_number"> max</xsl:if>]</xsl:when>
        <xsl:when test="type='7'">C</xsl:when>
        <xsl:when test="type='8'">Party possession detector [data: <xsl:value-of select="data"/>]</xsl:when>
        <xsl:when test="type='9'">Version detector</xsl:when>
      </xsl:choose>
    </xsl:if><xsl:text><![CDATA[<br>]]></xsl:text></xsl:for-each>
</xsl:template>

<!-- Write: tile content -->
<xsl:template name="tile-content">
  <xsl:param name="map_index"  select="0"/>
  <xsl:param name="map_width"  select="1"/>
  <xsl:param name="map_height" select="1"/>
  <xsl:param name="coord_y"    select="0"/>
  <xsl:param name="coord_x"    select="position()"/>
  
  <xsl:if test="count(/dungeon/objects/texts/text[@index=current()/items/text/@index and @visible=1])&gt;0
                or count(items/miscellaneous)&gt;0
                or count(items/cloth)&gt;0
                or count(items/potion)&gt;0
                or count(items/weapon)&gt;0
                or count(items/container)&gt;0
                or count(items/scroll)&gt;0
                or count(items/creature)&gt;0
                or count(items/actuator[type='127'])&gt;0">
    <tr>
    <td valign="top">(<xsl:value-of select="$coord_x"/>;<xsl:value-of select="$coord_y"/>)</td>
    <td><xsl:call-template name="tile-content-cell"/></td>
    </tr>
  </xsl:if>

</xsl:template>

<xsl:template name="tile-content-cell">
    <xsl:for-each select="items/actuator[type='127']">
      (champion)&#160;<xsl:call-template name="change-case">
        <xsl:with-param name="item" select="$doc_xml/dungeon/champions/champion[@type=current()/data]"/>
      </xsl:call-template><br/>
    </xsl:for-each>
    <xsl:for-each select="items/text">
      "<xsl:call-template name="change-case">
        <xsl:with-param name="item" select="/dungeon/objects/texts/text[@index=current()/@index]"/>
      </xsl:call-template>"<br/>
    </xsl:for-each>
    <xsl:for-each select="items/cloth">
      <xsl:variable name="cloth_type" select="/dungeon/objects/clothes/cloth[@index=current()/@index]/@type"/>      
      <xsl:value-of select="$doc_xml/dungeon/items/clothes/cloth[@type=$cloth_type]"/>
      <br/>
    </xsl:for-each>
    <xsl:for-each select="items/potion">
      <xsl:variable name="potion_type" select="/dungeon/objects/potions/potion[@index=current()/@index]/@type"/>      
      <xsl:value-of select="$doc_xml/dungeon/items/potions/potion[@type=$potion_type]"/>
      <br/>
    </xsl:for-each>
    <xsl:for-each select="items/weapon">
      <xsl:variable name="weapon_type" select="/dungeon/objects/weapons/weapon[@index=current()/@index]/@type"/>      
      <xsl:value-of select="$doc_xml/dungeon/items/weapons/weapon[@type=$weapon_type]"/>
      <br/>
    </xsl:for-each>
    <xsl:for-each select="items/container">
      <xsl:variable name="container_type" select="/dungeon/objects/containers/container[@index=current()/@index]/@type"/>      
      <xsl:value-of select="$doc_xml/dungeon/items/containers/container[@type=$container_type]"/>
      <br/><div class="tpContent"><xsl:call-template name="tile-content-cell"/></div>
    </xsl:for-each>
    <xsl:for-each select="items/scroll">
      Scroll&#160;("<xsl:call-template name="change-case">
        <xsl:with-param name="item" select="/dungeon/objects/texts/text[@index=/dungeon/objects/scrolls/scroll[@index=current()/@index]/@referred_text]/text()"/>
      </xsl:call-template>")
      <br/>
    </xsl:for-each>
    <xsl:for-each select="items/miscellaneous">
      <xsl:variable name="miscellaneous_type" select="/dungeon/objects/miscellaneouses/miscellaneous[@index=current()/@index]/@type"/>      
      <xsl:value-of select="$doc_xml/dungeon/items/miscellaneouses/miscellaneous[@type=$miscellaneous_type]"/>
      <br/>
    </xsl:for-each>
    <xsl:for-each select="items/creature">
      <xsl:variable name="creature_type" select="/dungeon/objects/creatures/creature[@index=current()/@index]/@type"/>      
      (creature)&#160;<xsl:value-of select="$doc_xml/dungeon/items/creatures/creature[@type=$creature_type]"/>
      <br/>
    </xsl:for-each>
</xsl:template>

<xsl:template name="tile-type-fullname">
  <xsl:param name="map_index" select="0"/>
  <xsl:choose>
  <xsl:when test="@type='0'">Wall</xsl:when>
  <xsl:when test="@type='1'">Floor</xsl:when>
  <xsl:when test="@type='2'"><xsl:choose>
      <xsl:when test="@is_invisible=1">Hidden </xsl:when>
    </xsl:choose><xsl:text>Pit</xsl:text>
  </xsl:when>
  <xsl:when test="@type='3'">Stairs</xsl:when>
  <xsl:when test="@type='4'"><xsl:variable name="door_graphic_type">
      <xsl:if test="/dungeon/objects/doors/door[@index=current()/items/door/@index]/@type='0'">
        <xsl:value-of select="/dungeon/maps/map[position()=($map_index+1)]/@graphics_door0"/>
      </xsl:if>
      <xsl:if test="/dungeon/objects/doors/door[@index=current()/items/door/@index]/@type='1'">
        <xsl:value-of select="/dungeon/maps/map[position()=($map_index+1)]/@graphics_door1"/>
      </xsl:if>
    </xsl:variable><xsl:value-of select="$doc_xml/dungeon/doors/door[@type=$door_graphic_type]"></xsl:value-of><xsl:text> Door</xsl:text>
  </xsl:when>
  <xsl:when test="@type='5'"><xsl:choose>
      <xsl:when test="@is_visible=0">Hidden </xsl:when>
    </xsl:choose><xsl:text>Teleporter</xsl:text>
  </xsl:when>
  <xsl:when test="@type='6'"><xsl:choose>
      <xsl:when test="@is_imaginary=0">Trick</xsl:when>
      <xsl:otherwise>False</xsl:otherwise>
    </xsl:choose><xsl:text> Wall</xsl:text>
  </xsl:when>
  <xsl:otherwise>Unknown</xsl:otherwise>
</xsl:choose></xsl:template>

<xsl:template name="random-decorations">
  <xsl:param name="map_index" select="0"/>
  <xsl:param name="x"         select="0"/>
  <xsl:param name="y"         select="0"/>
  <xsl:param name="count"     select="1"/>
  <xsl:if test="$count > 0">
    <td class="offset_rel">
      <xsl:call-template name="random-decoration">
        <xsl:with-param name="map_index" select="$map_index"/>
        <xsl:with-param name="x"         select="$x"/>
        <xsl:with-param name="y"         select="$y"/>
      </xsl:call-template>
    </td>
    <xsl:call-template name="random-decorations">
      <xsl:with-param name="map_index" select="$map_index"/>
      <xsl:with-param name="x"         select="$x + 1"/>
      <xsl:with-param name="y"         select="$y"/>
      <xsl:with-param name="count"     select="$count - 1"/>
    </xsl:call-template>
  </xsl:if>
</xsl:template>

<xsl:template name="random-decoration">
  <xsl:param name="map_index" select="0"/>
  <xsl:param name="x"         select="0"/>
  <xsl:param name="y"         select="0"/>
  <xsl:for-each select="/dungeon/maps/map[position()=($map_index+1)]/border_random_decorations/random_wall_decoration[@x=$x and @y=$y]">
    <xsl:variable name="decoration_graphic"      select="number(@graphic) - 1"/>
    <xsl:variable name="decoration_graphic_type" select="number(/dungeon/maps/map[position()=($map_index+1)]/wall_decoration_graphics/wall_decoration_graphic[@index=$decoration_graphic]/@type)"/>
    <xsl:if test="$decoration_graphic!=-1">
      <span>
        <xsl:attribute name="class">
          <xsl:text>wall_graphic_</xsl:text><xsl:value-of select="$decoration_graphic_type"/>
          <xsl:if test="@position='0'">
            <xsl:text>_north</xsl:text>
          </xsl:if>
          <xsl:if test="@position='1'">
            <xsl:text>_east</xsl:text>
          </xsl:if>
          <xsl:if test="@position='2'">
            <xsl:text>_south</xsl:text>
          </xsl:if>
          <xsl:if test="@position='3'">
            <xsl:text>_west</xsl:text>
          </xsl:if>
        </xsl:attribute>
      </span>
    </xsl:if>
  </xsl:for-each>
</xsl:template>

</xsl:stylesheet>
